@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 /* Device interface */
 
@@ -292,11 +292,13 @@ fz_java_device_begin_mask(fz_context *ctx, fz_device *dev, fz_rect rect, int lum
 }
 
 static void
-fz_java_device_end_mask(fz_context *ctx, fz_device *dev)
+fz_java_device_end_mask(fz_context *ctx, fz_device *dev, fz_function *tr)
 {
 	fz_java_device *jdev = (fz_java_device *)dev;
 	JNIEnv *env = jdev->env;
 
+	// TODO: pass transfer function
+
 	(*env)->CallVoidMethod(env, jdev->self, mid_Device_endMask);
 	if ((*env)->ExceptionCheck(env))
 		fz_throw_java(ctx, env);
@@ -378,7 +380,7 @@ fz_java_device_set_default_colorspaces(fz_context *ctx, fz_device *dev, fz_defau
 }
 
 static void
-fz_java_device_begin_structure(fz_context *ctx, fz_device *dev, fz_structure standard, const char *raw, int uid)
+fz_java_device_begin_structure(fz_context *ctx, fz_device *dev, fz_structure standard, const char *raw, int idx)
 {
 	fz_java_device *jdev = (fz_java_device *)dev;
 	JNIEnv *env = jdev->env;
@@ -388,7 +390,7 @@ fz_java_device_begin_structure(fz_context *ctx, fz_device *dev, fz_structure sta
 	if (!jraw || (*env)->ExceptionCheck(env))
 		fz_throw_java(ctx, env);
 
-	(*env)->CallVoidMethod(env, jdev->self, mid_Device_beginStructure, standard, jraw, uid);
+	(*env)->CallVoidMethod(env, jdev->self, mid_Device_beginStructure, standard, jraw, idx);
 	if ((*env)->ExceptionCheck(env))
 		fz_throw_java(ctx, env);
 }
