@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -637,6 +637,152 @@ FUN(PDFAnnotation_addInkListStrokeVertex)(JNIEnv *env, jobject self, float x, fl
 		jni_rethrow_void(env, ctx);
 }
 
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_hasCallout)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean has = JNI_FALSE;
+
+	fz_try(ctx)
+		has = pdf_annot_has_callout(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return has;
+}
+
+JNIEXPORT jint JNICALL
+FUN(PDFAnnotation_getCalloutStyle)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	enum pdf_line_ending s = 0;
+
+	fz_try(ctx)
+		s = pdf_annot_callout_style(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return s;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setCalloutStyle)(JNIEnv *env, jobject self, jint s)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_try(ctx)
+		pdf_set_annot_callout_style(ctx, annot, s);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jobject JNICALL
+FUN(PDFAnnotation_getCalloutPoint)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point p;
+
+	if (!ctx || !annot) return NULL;
+
+	fz_try(ctx)
+		p = pdf_annot_callout_point(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return (*env)->NewObject(env, cls_Point, mid_Point_init, p.x, p.y);
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setCalloutPoint)(JNIEnv *env, jobject self, jobject jp)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point p;
+
+	if (!ctx || !annot) return;
+
+	if (!jp) jni_throw_arg_void(env, "point must not be null");
+
+	p.x = (*env)->GetFloatField(env, jp, fid_Point_x);
+	p.y = (*env)->GetFloatField(env, jp, fid_Point_y);
+
+	fz_try(ctx)
+		pdf_set_annot_callout_point(ctx, annot, p);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jobject JNICALL
+FUN(PDFAnnotation_getCalloutLine)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point points[3] = { 0 };
+	jobject jline = NULL;
+	jobject jpoint = NULL;
+	int i, n;
+
+	if (!ctx || !annot) return NULL;
+
+	fz_try(ctx)
+		pdf_annot_callout_line(ctx, annot, points, &n);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	if (n == 0)
+		return NULL;
+
+	jline = (*env)->NewObjectArray(env, n, cls_Point, NULL);
+	if (!jline || (*env)->ExceptionCheck(env)) return NULL;
+
+	for (i = 0; i < n; i++)
+	{
+		jpoint = (*env)->NewObject(env, cls_Point, mid_Point_init, points[i].x, points[i].y);
+		if (!jpoint || (*env)->ExceptionCheck(env)) return NULL;
+		(*env)->SetObjectArrayElement(env, jline, i, jpoint);
+		if ((*env)->ExceptionCheck(env)) return NULL;
+		(*env)->DeleteLocalRef(env, jpoint);
+	}
+
+	return jline;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setCalloutLineNative)(JNIEnv *env, jobject self, jint n, jobject ja, jobject jb, jobject jc)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point line[3];
+
+	if (!ctx || !annot) return;
+
+	if (n >= 2 && !ja) jni_throw_arg_void(env, "points must not be null");
+	if (n >= 2 && !jb) jni_throw_arg_void(env, "points must not be null");
+	if (n >= 3 && !jc) jni_throw_arg_void(env, "points must not be null");
+
+	if (n >= 2) {
+		line[0].x = (*env)->GetFloatField(env, ja, fid_Point_x);
+		line[0].y = (*env)->GetFloatField(env, ja, fid_Point_y);
+		line[1].x = (*env)->GetFloatField(env, jb, fid_Point_x);
+		line[1].y = (*env)->GetFloatField(env, jb, fid_Point_y);
+	}
+	if (n >= 3) {
+		line[2].x = (*env)->GetFloatField(env, jc, fid_Point_x);
+		line[2].y = (*env)->GetFloatField(env, jc, fid_Point_y);
+	}
+
+	fz_try(ctx)
+	{
+		if (n == 0 || n == 2 || n == 3)
+			pdf_set_annot_callout_line(ctx, annot, line, n);
+	}
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
 JNIEXPORT void JNICALL
 FUN(PDFAnnotation_eventEnter)(JNIEnv *env, jobject self)
 {
@@ -1305,7 +1451,7 @@ FUN(PDFAnnotation_hasLine)(JNIEnv *env, jobject self)
 }
 
 JNIEXPORT void JNICALL
-FUN(PDFAnnotation_setFileSpecification)(JNIEnv *env, jobject self, jobject jfs)
+FUN(PDFAnnotation_setFilespec)(JNIEnv *env, jobject self, jobject jfs)
 {
 	fz_context *ctx = get_context(env);
 	pdf_annot *annot = from_PDFAnnotation(env, self);
@@ -1318,7 +1464,7 @@ FUN(PDFAnnotation_setFileSpecification)(JNIEnv *env, jobject self, jobject jfs)
 }
 
 JNIEXPORT jobject JNICALL
-FUN(PDFAnnotation_getFileSpecification)(JNIEnv *env, jobject self)
+FUN(PDFAnnotation_getFilespec)(JNIEnv *env, jobject self)
 {
 	fz_context *ctx = get_context(env);
 	pdf_annot *annot = from_PDFAnnotation(env, self);
@@ -1537,7 +1683,7 @@ FUN(PDFAnnotation_setBorderEffectIntensity)(JNIEnv *env, jobject self, jfloat in
 }
 
 JNIEXPORT jboolean JNICALL
-FUN(PDFAnnotation_hasFileSpecification)(JNIEnv *env, jobject self)
+FUN(PDFAnnotation_hasFilespec)(JNIEnv *env, jobject self)
 {
 	fz_context *ctx = get_context(env);
 	pdf_annot *annot = from_PDFAnnotation(env, self);
@@ -1614,3 +1760,253 @@ FUN(PDFAnnotation_hasRect)(JNIEnv *env, jobject self)
 
 	return has;
 }
+
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_hasIntent)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean has = JNI_FALSE;
+
+	fz_try(ctx)
+		has = pdf_annot_has_intent(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return has;
+}
+
+JNIEXPORT jint JNICALL
+FUN(PDFAnnotation_getIntent)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	enum pdf_intent intent = PDF_ANNOT_IT_DEFAULT;
+
+	if (!ctx || !annot) return PDF_ANNOT_IT_DEFAULT;
+
+	fz_try(ctx)
+		intent = pdf_annot_intent(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return intent;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setIntent)(JNIEnv *env, jobject self, jint intent)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_intent(ctx, annot, intent);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_hasPopup)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean has = JNI_FALSE;
+
+	fz_try(ctx)
+		has = pdf_annot_has_popup(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return has;
+}
+
+JNIEXPORT jobject JNICALL
+FUN(PDFAnnotation_getPopup)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_rect rect;
+
+	if (!ctx || !annot) return JNI_FALSE;
+
+	fz_try(ctx)
+		rect = pdf_annot_popup(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return to_Rect_safe(ctx, env, rect);
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setPopup)(JNIEnv *env, jobject self, jobject jrect)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_rect rect = from_Rect(env, jrect);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_popup(ctx, annot, rect);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jfloat JNICALL
+FUN(PDFAnnotation_getLineLeader)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jfloat v;
+
+	if (!ctx || !annot) return 0;
+
+	fz_try(ctx)
+		v = pdf_annot_line_leader(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return v;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setLineLeader)(JNIEnv *env, jobject self, jfloat length)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_line_leader(ctx, annot, length);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+
+JNIEXPORT jfloat JNICALL
+FUN(PDFAnnotation_getLineLeaderExtension)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jfloat v;
+
+	if (!ctx || !annot) return 0;
+
+	fz_try(ctx)
+		v = pdf_annot_line_leader_extension(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return v;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setLineLeaderExtension)(JNIEnv *env, jobject self, jfloat extension)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_line_leader_extension(ctx, annot, extension);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jfloat JNICALL
+FUN(PDFAnnotation_getLineLeaderOffset)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jfloat v;
+
+	if (!ctx || !annot) return 0;
+
+	fz_try(ctx)
+		v = pdf_annot_line_leader_offset(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return v;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setLineLeaderOffset)(JNIEnv *env, jobject self, jfloat offset)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_line_leader_offset(ctx, annot, offset);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jboolean JNICALL
+FUN(PDFAnnotation_getLineCaption)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	jboolean v;
+
+	if (!ctx || !annot) return 0;
+
+	fz_try(ctx)
+		v = pdf_annot_line_caption(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return v;
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setLineCaption)(JNIEnv *env, jobject self, jboolean caption)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_line_caption(ctx, annot, caption);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
+JNIEXPORT jobject JNICALL
+FUN(PDFAnnotation_getLineCaptionOffset)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point offset;;
+
+	if (!ctx || !annot) return NULL;
+
+	fz_try(ctx)
+		offset = pdf_annot_line_caption_offset(ctx, annot);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return to_Point_safe(ctx, env, offset);
+}
+
+JNIEXPORT void JNICALL
+FUN(PDFAnnotation_setLeaderLines)(JNIEnv *env, jobject self, jobject joffset)
+{
+	fz_context *ctx = get_context(env);
+	pdf_annot *annot = from_PDFAnnotation(env, self);
+	fz_point offset = from_Point(env, joffset);
+
+	if (!ctx || !annot) return;
+
+	fz_try(ctx)
+		pdf_set_annot_line_caption_offset(ctx, annot, offset);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
