@@ -1,42 +1,35 @@
-#include "fitz-imp.h"
+// Copyright (C) 2004-2024 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
+#include "mupdf/fitz.h"
+
+#include "context-imp.h"
 
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
 #include <time.h>
 
-struct fz_id_context_s
-{
-	int refs;
-	int id;
-};
-
-static void
-fz_drop_id_context(fz_context *ctx)
-{
-	if (!ctx)
-		return;
-	if (fz_drop_imp(ctx, ctx->id, &ctx->id->refs))
-		fz_free(ctx, ctx->id);
-}
-
-static void
-fz_new_id_context(fz_context *ctx)
-{
-	ctx->id = fz_malloc_struct(ctx, fz_id_context);
-	ctx->id->refs = 1;
-	ctx->id->id = 0;
-}
-
-static fz_id_context *
-fz_keep_id_context(fz_context *ctx)
-{
-	if (!ctx)
-		return NULL;
-	return fz_keep_imp(ctx, ctx->id, &ctx->id->refs);
-}
-
-struct fz_style_context_s
+struct fz_style_context
 {
 	int refs;
 	char *user_css;
@@ -155,82 +148,45 @@ fz_drop_context(fz_context *ctx)
 	if (!ctx)
 		return;
 
+	if (ctx->error.errcode)
+	{
+		fz_flush_warnings(ctx);
+		fz_warn(ctx, "UNHANDLED EXCEPTION!");
+		fz_report_error(ctx);
+#ifdef CLUSTER
+		abort();
+#endif
+	}
+
 	/* Other finalisation calls go here (in reverse order) */
 	fz_drop_document_handler_context(ctx);
+	fz_drop_archive_handler_context(ctx);
 	fz_drop_glyph_cache_context(ctx);
 	fz_drop_store_context(ctx);
-	fz_drop_aa_context(ctx);
 	fz_drop_style_context(ctx);
 	fz_drop_tuning_context(ctx);
 	fz_drop_colorspace_context(ctx);
-	fz_drop_cmm_context(ctx);
 	fz_drop_font_context(ctx);
-	fz_drop_id_context(ctx);
-	fz_drop_output_context(ctx);
 
-	if (ctx->warn)
-	{
-		fz_flush_warnings(ctx);
-		fz_free(ctx, ctx->warn);
-	}
+	fz_flush_warnings(ctx);
 
-	if (ctx->error)
-	{
-		assert(ctx->error->top == ctx->error->stack - 1);
-		fz_free(ctx, ctx->error);
-	}
+	assert(ctx->error.top == ctx->error.stack_base);
 
 	/* Free the context itself */
-	ctx->alloc->free(ctx->alloc->user, ctx);
+	ctx->alloc.free(ctx->alloc.user, ctx);
 }
 
-/* Allocate new context structure, and initialise allocator, and sections
- * that aren't shared between contexts.
- */
-static fz_context *
-new_context_phase1(const fz_alloc_context *alloc, const fz_locks_context *locks)
+static void
+fz_init_error_context(fz_context *ctx)
 {
-	fz_context *ctx;
+#define ALIGN(addr, align)  ((((intptr_t)(addr)) + (align-1)) & ~(align-1))
+	ctx->error.stack_base = (fz_error_stack_slot *)ALIGN(ctx->error.stack, FZ_JMPBUF_ALIGN);
+	ctx->error.top = ctx->error.stack_base;
+	ctx->error.errcode = FZ_ERROR_NONE;
+	ctx->error.message[0] = 0;
 
-	ctx = alloc->malloc(alloc->user, sizeof(fz_context));
-	if (!ctx)
-		return NULL;
-	memset(ctx, 0, sizeof *ctx);
-	ctx->user = NULL;
-	ctx->alloc = alloc;
-	ctx->locks = *locks;
-
-	ctx->glyph_cache = NULL;
-
-	ctx->error = Memento_label(fz_malloc_no_throw(ctx, sizeof(fz_error_context)), "fz_error_context");
-	if (!ctx->error)
-		goto cleanup;
-	ctx->error->top = ctx->error->stack - 1;
-	ctx->error->errcode = FZ_ERROR_NONE;
-	ctx->error->message[0] = 0;
-
-	ctx->warn = Memento_label(fz_malloc_no_throw(ctx, sizeof(fz_warn_context)), "fz_warn_context");
-	if (!ctx->warn)
-		goto cleanup;
-	ctx->warn->message[0] = 0;
-	ctx->warn->count = 0;
-
-	/* New initialisation calls for context entries go here */
-	fz_try(ctx)
-	{
-		fz_new_aa_context(ctx);
-	}
-	fz_catch(ctx)
-	{
-		goto cleanup;
-	}
-
-	return ctx;
-
-cleanup:
-	fprintf(stderr, "cannot create context (phase 1)\n");
-	fz_drop_context(ctx);
-	return NULL;
+	ctx->warn.message[0] = 0;
+	ctx->warn.count = 0;
 }
 
 fz_context *
@@ -250,27 +206,40 @@ fz_new_context_imp(const fz_alloc_context *alloc, const fz_locks_context *locks,
 	if (!locks)
 		locks = &fz_locks_default;
 
-	ctx = new_context_phase1(alloc, locks);
+	ctx = Memento_label(alloc->malloc(alloc->user, sizeof(fz_context)), "fz_context");
 	if (!ctx)
+	{
+		fprintf(stderr, "cannot create context (phase 1)\n");
 		return NULL;
+	}
+	memset(ctx, 0, sizeof *ctx);
+
+	ctx->user = NULL;
+	ctx->alloc = *alloc;
+	ctx->locks = *locks;
+
+	ctx->error.print = fz_default_error_callback;
+	ctx->warn.print = fz_default_warning_callback;
+
+	fz_init_error_context(ctx);
+	fz_init_aa_context(ctx);
+	fz_init_random_context(ctx);
 
 	/* Now initialise sections that are shared */
 	fz_try(ctx)
 	{
-		fz_new_output_context(ctx);
 		fz_new_store_context(ctx, max_store);
 		fz_new_glyph_cache_context(ctx);
-		fz_new_cmm_context(ctx);
 		fz_new_colorspace_context(ctx);
 		fz_new_font_context(ctx);
-		fz_new_id_context(ctx);
 		fz_new_document_handler_context(ctx);
+		fz_new_archive_handler_context(ctx);
 		fz_new_style_context(ctx);
 		fz_new_tuning_context(ctx);
-		fz_init_random_context(ctx);
 	}
 	fz_catch(ctx)
 	{
+		fz_report_error(ctx);
 		fprintf(stderr, "cannot create context (phase 2)\n");
 		fz_drop_context(ctx);
 		return NULL;
@@ -281,67 +250,36 @@ fz_new_context_imp(const fz_alloc_context *alloc, const fz_locks_context *locks,
 fz_context *
 fz_clone_context(fz_context *ctx)
 {
+	fz_context *new_ctx;
+
 	/* We cannot safely clone the context without having locking/
 	 * unlocking functions. */
 	if (ctx == NULL || (ctx->locks.lock == fz_locks_default.lock && ctx->locks.unlock == fz_locks_default.unlock))
 		return NULL;
-	return fz_clone_context_internal(ctx);
-}
 
-fz_context *
-fz_clone_context_internal(fz_context *ctx)
-{
-	fz_context *new_ctx;
-
-	if (ctx == NULL || ctx->alloc == NULL)
-		return NULL;
-
-	new_ctx = new_context_phase1(ctx->alloc, &ctx->locks);
+	new_ctx = ctx->alloc.malloc(ctx->alloc.user, sizeof(fz_context));
 	if (!new_ctx)
 		return NULL;
 
-	/* Inherit AA defaults from old context. */
-	fz_copy_aa_context(new_ctx, ctx);
+	/* First copy old context, including pointers to shared contexts */
+	memcpy(new_ctx, ctx, sizeof (fz_context));
 
-	/* Keep thread lock checking happy by copying pointers first and locking under new context */
-	new_ctx->output = ctx->output;
-	new_ctx->output = fz_keep_output_context(new_ctx);
-	new_ctx->user = ctx->user;
-	new_ctx->store = ctx->store;
-	new_ctx->store = fz_keep_store_context(new_ctx);
-	new_ctx->glyph_cache = ctx->glyph_cache;
-	new_ctx->glyph_cache = fz_keep_glyph_cache(new_ctx);
-	new_ctx->colorspace = ctx->colorspace;
-	new_ctx->colorspace = fz_keep_colorspace_context(new_ctx);
-	fz_new_cmm_context(new_ctx);
-	new_ctx->font = ctx->font;
-	new_ctx->font = fz_keep_font_context(new_ctx);
-	new_ctx->style = ctx->style;
-	new_ctx->style = fz_keep_style_context(new_ctx);
-	new_ctx->id = ctx->id;
-	new_ctx->id = fz_keep_id_context(new_ctx);
-	new_ctx->tuning = ctx->tuning;
-	new_ctx->tuning = fz_keep_tuning_context(new_ctx);
-	memcpy(new_ctx->seed48, ctx->seed48, sizeof ctx->seed48);
-	new_ctx->handler = ctx->handler;
-	new_ctx->handler = fz_keep_document_handler_context(new_ctx);
+	/* Reset error context to initial state. */
+	fz_init_error_context(new_ctx);
+
+	/* Then keep lock checking happy by keeping shared contexts with new context */
+	fz_keep_document_handler_context(new_ctx);
+	fz_keep_archive_handler_context(new_ctx);
+	fz_keep_style_context(new_ctx);
+	fz_keep_tuning_context(new_ctx);
+	fz_keep_font_context(new_ctx);
+	fz_keep_colorspace_context(new_ctx);
+	fz_keep_store_context(new_ctx);
+	fz_keep_glyph_cache(new_ctx);
 
 	return new_ctx;
 }
 
-int
-fz_gen_id(fz_context *ctx)
-{
-	int id;
-	fz_lock(ctx, FZ_LOCK_ALLOC);
-	/* We'll never wrap around in normal use, but if we do, then avoid 0. */
-	do
-		id = ++ctx->id->id;
-	while (id == 0);
-	fz_unlock(ctx, FZ_LOCK_ALLOC);
-	return id;
-}
-
 void fz_set_user_context(fz_context *ctx, void *user)
 {
 	if (ctx != NULL)
