@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "draw-imp.h"
 
@@ -119,7 +141,7 @@ fz_insert_gel_raw(fz_context *ctx, fz_rasterizer *ras, int x0, int y0, int x1, i
 
 	if (gel->len + 1 == gel->cap) {
 		int new_cap = gel->cap * 2;
-		gel->edges = fz_resize_array(ctx, gel->edges, new_cap, sizeof(fz_edge));
+		gel->edges = fz_realloc_array(ctx, gel->edges, new_cap, fz_edge);
 		gel->cap = new_cap;
 	}
 
@@ -382,7 +404,7 @@ insert_active(fz_context *ctx, fz_gel *gel, int y, int *e_)
 		do {
 			if (gel->alen + 1 == gel->acap) {
 				int newcap = gel->acap + 64;
-				fz_edge **newactive = fz_resize_array(ctx, gel->active, newcap, sizeof(fz_edge*));
+				fz_edge **newactive = fz_realloc_array(ctx, gel->active, newcap, fz_edge*);
 				gel->active = newactive;
 				gel->acap = newcap;
 			}
@@ -536,7 +558,7 @@ static inline void
 blit_aa(fz_pixmap *dst, int x, int y, unsigned char *mp, int w, unsigned char *color, void *fn, fz_overprint *eop)
 {
 	unsigned char *dp;
-	dp = dst->samples + (unsigned int)((y - dst->y) * dst->stride + (x - dst->x) * dst->n);
+	dp = dst->samples + (y - dst->y) * (size_t)dst->stride + (x - dst->x) * (size_t)dst->n;
 	if (color)
 		(*(fz_span_color_painter_t *)fn)(dp, mp, dst->n, w, color, dst->alpha, eop);
 	else
@@ -579,8 +601,8 @@ fz_scan_convert_aa(fz_context *ctx, fz_gel *gel, int eofill, const fz_irect *cli
 		fz_free(ctx, gel->deltas);
 		gel->alphas = NULL;
 		gel->deltas = NULL;
-		alphas = gel->alphas = fz_malloc_array(ctx, bcap, sizeof (unsigned char));
-		deltas = gel->deltas = fz_malloc_array(ctx, bcap, sizeof (int));
+		gel->alphas = Memento_label(fz_malloc_array(ctx, bcap, unsigned char), "gel_alphas");
+		gel->deltas = Memento_label(fz_malloc_array(ctx, bcap, int), "gel_deltas");
 	}
 	alphas = gel->alphas;
 	deltas = gel->deltas;
@@ -744,7 +766,7 @@ blit_sharp(int x0, int x1, int y, const fz_irect *clip, fz_pixmap *dst, unsigned
 	x1 = fz_clampi(x1, dst->x, dst->x + dst->w);
 	if (x0 < x1)
 	{
-		dp = dst->samples + (unsigned int)((y - dst->y) * dst->stride + (x0 - dst->x) * dst->n);
+		dp = dst->samples + (y - dst->y) * (size_t)dst->stride + (x0 - dst->x) * (size_t)dst->n;
 		if (color)
 			(*fn)(dp, dst->n, x1 - x0, color, da, eop);
 		else
@@ -854,7 +876,6 @@ fz_convert_gel(fz_context *ctx, fz_rasterizer *rast, int eofill, const fz_irect
 			fn = (void *)fz_get_span_color_painter(dst->n, dst->alpha, color, eop);
 		else
 			fn = (void *)fz_get_span_painter(dst->alpha, 1, 0, 255, eop);
-		assert(fn);
 		if (fn == NULL)
 			return;
 		fz_scan_convert_aa(ctx, gel, eofill, clip, dst, color, fn, eop);
@@ -893,11 +914,11 @@ fz_new_gel(fz_context *ctx)
 		gel->edges = NULL;
 		gel->cap = 512;
 		gel->len = 0;
-		gel->edges = fz_malloc_array(ctx, gel->cap, sizeof(fz_edge));
+		gel->edges = Memento_label(fz_malloc_array(ctx, gel->cap, fz_edge), "gel_edges");
 
 		gel->acap = 64;
 		gel->alen = 0;
-		gel->active = fz_malloc_array(ctx, gel->acap, sizeof(fz_edge*));
+		gel->active = Memento_label(fz_malloc_array(ctx, gel->acap, fz_edge*), "gel_active");
 	}
 	fz_catch(ctx)
 	{
