@@ -1,29 +1,51 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 
+#include <string.h>
+#include <assert.h>
+
 /*
-Simple hashtable with open addressing linear probe.
-Unlike text book examples, removing entries works
-correctly in this implementation, so it won't start
-exhibiting bad behaviour if entries are inserted
-and removed frequently.
+	Simple hashtable with open addressing linear probe.
+	Unlike text book examples, removing entries works
+	correctly in this implementation, so it won't start
+	exhibiting bad behaviour if entries are inserted
+	and removed frequently.
 */
 
-enum { MAX_KEY_LEN = 48 };
-typedef struct fz_hash_entry_s fz_hash_entry;
-
-struct fz_hash_entry_s
+typedef struct
 {
-	unsigned char key[MAX_KEY_LEN];
+	unsigned char key[FZ_HASH_TABLE_KEY_LENGTH];
 	void *val;
-};
+} fz_hash_entry;
 
-struct fz_hash_table_s
+struct fz_hash_table
 {
 	int keylen;
 	int size;
 	int load;
 	int lock; /* -1 or the lock used to protect this hash table */
-	fz_hash_table_drop_fn drop_val;
+	fz_hash_table_drop_fn *drop_val;
 	fz_hash_entry *ents;
 };
 
@@ -44,11 +66,12 @@ static unsigned hash(const unsigned char *s, int len)
 }
 
 fz_hash_table *
-fz_new_hash_table(fz_context *ctx, int initialsize, int keylen, int lock, fz_hash_table_drop_fn drop_val)
+fz_new_hash_table(fz_context *ctx, int initialsize, int keylen, int lock, fz_hash_table_drop_fn *drop_val)
 {
 	fz_hash_table *table;
 
-	assert(keylen <= MAX_KEY_LEN);
+	if (keylen > FZ_HASH_TABLE_KEY_LENGTH)
+		fz_throw(ctx, FZ_ERROR_ARGUMENT, "hash table key length too large");
 
 	table = fz_malloc_struct(ctx, fz_hash_table);
 	table->keylen = keylen;
@@ -58,7 +81,7 @@ fz_new_hash_table(fz_context *ctx, int initialsize, int keylen, int lock, fz_has
 	table->drop_val = drop_val;
 	fz_try(ctx)
 	{
-		table->ents = fz_malloc_array(ctx, table->size, sizeof(fz_hash_entry));
+		table->ents = Memento_label(fz_malloc_array(ctx, table->size, fz_hash_entry), "hash_entries");
 		memset(table->ents, 0, sizeof(fz_hash_entry) * table->size);
 	}
 	fz_catch(ctx)
@@ -118,7 +141,6 @@ do_hash_insert(fz_context *ctx, fz_hash_table *table, const void *key, void *val
 		if (memcmp(key, ents[pos].key, table->keylen) == 0)
 		{
 			/* This is legal, but should rarely happen. */
-			fz_warn(ctx, "assert: overwrite hash slot");
 			return ents[pos].val;
 		}
 
@@ -145,7 +167,7 @@ fz_resize_hash(fz_context *ctx, fz_hash_table *table, int newsize)
 
 	if (table->lock == FZ_LOCK_ALLOC)
 		fz_unlock(ctx, table->lock);
-	newents = fz_malloc_array_no_throw(ctx, newsize, sizeof(fz_hash_entry));
+	newents = Memento_label(fz_malloc_no_throw(ctx, newsize * sizeof (fz_hash_entry)), "hash_entries");
 	if (table->lock == FZ_LOCK_ALLOC)
 		fz_lock(ctx, table->lock);
 	if (table->lock >= 0)
@@ -162,7 +184,7 @@ fz_resize_hash(fz_context *ctx, fz_hash_table *table, int newsize)
 		}
 	}
 	if (newents == NULL)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "hash table resize failed; out of memory (%d entries)", newsize);
+		fz_throw(ctx, FZ_ERROR_SYSTEM, "hash table resize failed; out of memory (%d entries)", newsize);
 	table->ents = newents;
 	memset(table->ents, 0, sizeof(fz_hash_entry) * newsize);
 	table->size = newsize;
@@ -214,7 +236,7 @@ fz_hash_insert(fz_context *ctx, fz_hash_table *table, const void *key, void *val
 }
 
 static void
-do_removal(fz_context *ctx, fz_hash_table *table, const void *key, unsigned hole)
+do_removal(fz_context *ctx, fz_hash_table *table, unsigned hole)
 {
 	fz_hash_entry *ents = table->ents;
 	unsigned size = table->size;
@@ -269,7 +291,7 @@ fz_hash_remove(fz_context *ctx, fz_hash_table *table, const void *key)
 
 		if (memcmp(key, ents[pos].key, table->keylen) == 0)
 		{
-			do_removal(ctx, table, key, pos);
+			do_removal(ctx, table, pos);
 			return;
 		}
 
@@ -280,31 +302,28 @@ fz_hash_remove(fz_context *ctx, fz_hash_table *table, const void *key)
 }
 
 void
-fz_print_hash(fz_context *ctx, fz_output *out, fz_hash_table *table)
+fz_hash_for_each(fz_context *ctx, fz_hash_table *table, void *state, fz_hash_table_for_each_fn *callback)
 {
-	fz_print_hash_details(ctx, out, table, NULL, 0);
+	int i;
+	for (i = 0; i < table->size; ++i)
+		if (table->ents[i].val)
+			callback(ctx, state, table->ents[i].key, table->keylen, table->ents[i].val);
 }
 
 void
-fz_print_hash_details(fz_context *ctx, fz_output *out, fz_hash_table *table, void (*details)(fz_context*,fz_output*,void*), int compact)
+fz_hash_filter(fz_context *ctx, fz_hash_table *table, void *state, fz_hash_table_filter_fn *callback)
 {
-	int i, k;
-
-	fz_write_printf(ctx, out, "cache load %d / %d\n", table->load, table->size);
-
-	for (i = 0; i < table->size; i++)
+	int i;
+restart:
+	for (i = 0; i < table->size; ++i)
 	{
-		if (!table->ents[i].val && !compact)
-			fz_write_printf(ctx, out, "table %04d: empty\n", i);
-		else if (table->ents[i].val)
+		if (table->ents[i].val)
 		{
-			fz_write_printf(ctx, out, "table %04d: key=", i);
-			for (k = 0; k < MAX_KEY_LEN; k++)
-				fz_write_printf(ctx, out, "%02x", ((unsigned char*)table->ents[i].key)[k]);
-			if (details)
-				details(ctx, out, table->ents[i].val);
-			else
-				fz_write_printf(ctx, out, " val=$%p\n", table->ents[i].val);
+			if (callback(ctx, state, table->ents[i].key, table->keylen, table->ents[i].val))
+			{
+				do_removal(ctx, table, i);
+				goto restart; /* we may have moved some slots around, so just restart the scan */
+			}
 		}
 	}
 }
