@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -65,6 +65,13 @@ typedef struct pdf_filter_gstate
 	pdf_text_state text;
 } pdf_filter_gstate;
 
+typedef enum
+{
+	NO_CLIP_OP,
+	CLIP_W,
+	CLIP_Wstar
+} clip_op_t;
+
 typedef struct filter_gstate
 {
 	struct filter_gstate *next;
@@ -72,6 +79,7 @@ typedef struct filter_gstate
 	fz_rect clip_rect;
 	pdf_filter_gstate pending;
 	pdf_filter_gstate sent;
+	clip_op_t clip_op;
 } filter_gstate;
 
 typedef struct
@@ -130,7 +138,6 @@ typedef struct
 	/* Has any marking text been removed so far this text object? */
 	int text_removed;
 	pdf_filter_options *global_options;
-	/* path is only constructed if we are using culling. */
 	fz_path *path;
 } pdf_sanitize_processor;
 
@@ -571,8 +578,9 @@ filter_show_char(fz_context *ctx, pdf_sanitize_processor *p, int cid, int *unico
 	int ucsbuf[PDF_MRANGE_CAP];
 	int ucslen;
 	int remove = 0;
+	float adv;
 
-	(void)pdf_tos_make_trm(ctx, &p->tos, &gstate->pending.text, fontdesc, cid, &trm);
+	(void)pdf_tos_make_trm(ctx, &p->tos, &gstate->pending.text, fontdesc, cid, &trm, &adv);
 
 	ucslen = 0;
 	if (fontdesc->to_unicode)
@@ -1191,7 +1199,7 @@ pdf_filter_gs_ca(fz_context *ctx, pdf_processor *proc, float alpha)
 }
 
 static void
-pdf_filter_gs_SMask(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, float *bc, int luminosity, pdf_obj *tr)
+pdf_filter_gs_SMask(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, fz_colorspace *smask_cs, float *bc, int luminosity, pdf_obj *tr)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
 
@@ -1199,7 +1207,7 @@ pdf_filter_gs_SMask(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, float
 		return;
 
 	if (p->chain->op_gs_SMask)
-		p->chain->op_gs_SMask(ctx, p->chain, smask, bc, luminosity, tr);
+		p->chain->op_gs_SMask(ctx, p->chain, smask, smask_cs, bc, luminosity, tr);
 }
 
 static void
@@ -1221,9 +1229,6 @@ pdf_filter_q(fz_context *ctx, pdf_processor *proc)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
 
-	if (fz_is_empty_rect(p->gstate->clip_rect))
-		return;
-
 	filter_push(ctx, p);
 }
 
@@ -1262,12 +1267,6 @@ pdf_filter_m(fz_context *ctx, pdf_processor *proc, float x, float y)
 		return;
 
 	fz_moveto(ctx, p->path, x, y);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_m)
-		p->chain->op_m(ctx, p->chain, x, y);
 }
 
 static void
@@ -1279,12 +1278,6 @@ pdf_filter_l(fz_context *ctx, pdf_processor *proc, float x, float y)
 		return;
 
 	fz_lineto(ctx, p->path, x, y);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_l)
-		p->chain->op_l(ctx, p->chain, x, y);
 }
 
 static void
@@ -1296,12 +1289,6 @@ pdf_filter_c(fz_context *ctx, pdf_processor *proc, float x1, float y1, float x2,
 		return;
 
 	fz_curveto(ctx, p->path, x1, y1, x2, y2, x3, y3);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_c)
-		p->chain->op_c(ctx, p->chain, x1, y1, x2, y2, x3, y3);
 }
 
 static void
@@ -1313,12 +1300,6 @@ pdf_filter_v(fz_context *ctx, pdf_processor *proc, float x2, float y2, float x3,
 		return;
 
 	fz_curvetov(ctx, p->path, x2, y2, x3, y3);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_v)
-		p->chain->op_v(ctx, p->chain, x2, y2, x3, y3);
 }
 
 static void
@@ -1330,12 +1311,6 @@ pdf_filter_y(fz_context *ctx, pdf_processor *proc, float x1, float y1, float x3,
 		return;
 
 	fz_curvetoy(ctx, p->path, x1, y1, x3, y3);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_y)
-		p->chain->op_y(ctx, p->chain, x1, y1, x3, y3);
 }
 
 static void
@@ -1347,12 +1322,6 @@ pdf_filter_h(fz_context *ctx, pdf_processor *proc)
 		return;
 
 	fz_closepath(ctx, p->path);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_h)
-		p->chain->op_h(ctx, p->chain);
 }
 
 static void
@@ -1364,12 +1333,6 @@ pdf_filter_re(fz_context *ctx, pdf_processor *proc, float x, float y, float w, f
 		return;
 
 	fz_rectto(ctx, p->path, x, y, x+w, y+h);
-	if (p->options->culler)
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_re)
-		p->chain->op_re(ctx, p->chain, x, y, w, h);
 }
 
 /* path painting */
@@ -1444,21 +1407,23 @@ typedef struct
 	fz_matrix ctm;
 	int any_sent;
 	int type;
+	int flush;
 } segmenter_data_t;
 
+static const fz_path_walker cull_replay_walker = {
+	cull_replay_moveto,
+	cull_replay_lineto,
+	cull_replay_curveto,
+	cull_replay_closepath,
+	NULL /* quad */,
+	cull_replay_curvetov,
+	cull_replay_curvetoy,
+	cull_replay_rectto
+};
+
 static void
 end_segment(fz_context *ctx, segmenter_data_t *sd)
 {
-	static const fz_path_walker walker = {
-		cull_replay_moveto,
-		cull_replay_lineto,
-		cull_replay_curveto,
-		cull_replay_closepath,
-		NULL /* quad */,
-		cull_replay_curvetov,
-		cull_replay_curvetoy,
-		cull_replay_rectto
-	};
 	fz_rect r;
 	const fz_stroke_state *st;
 
@@ -1475,7 +1440,13 @@ end_segment(fz_context *ctx, segmenter_data_t *sd)
 	else
 	{
 		/* Better send this segment then! */
-		fz_walk_path(ctx, sd->segment, &walker, sd->p);
+		/* Flush, just once, at the last possible minute. */
+		if (sd->flush)
+		{
+			filter_flush(ctx, sd->p, sd->flush);
+			sd->flush = 0;
+		}
+		fz_walk_path(ctx, sd->segment, &cull_replay_walker, sd->p);
 		sd->any_sent = 1;
 	}
 	fz_drop_path(ctx, sd->segment);
@@ -1542,9 +1513,8 @@ cull_segmenter_rectto(fz_context *ctx, void *arg, float x1, float y1, float x2,
 	fz_rectto(ctx, sd->segment, x1, y1, x2, y2);
 }
 
-
 static int
-cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type)
+cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type, int flush)
 {
 	segmenter_data_t sd = { 0 };
 	static const fz_path_walker segmenter = {
@@ -1560,6 +1530,29 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type)
 
 	if (p->options->culler == NULL)
 	{
+		filter_flush(ctx, p, flush);
+		fz_walk_path(ctx, p->path, &cull_replay_walker, p);
+
+		if (p->gstate->clip_op != NO_CLIP_OP)
+		{
+			fz_rect r;
+
+			/* ctm has always been flushed by now. */
+			r = fz_bound_path(ctx, p->path, NULL, p->gstate->sent.ctm);
+			p->gstate->clip_rect = fz_intersect_rect(p->gstate->clip_rect, r);
+			if (p->gstate->clip_op == CLIP_W)
+			{
+				if (p->chain->op_W)
+					p->chain->op_W(ctx, p->chain);
+			}
+			else
+			{
+				if (p->chain->op_Wstar)
+					p->chain->op_Wstar(ctx, p->chain);
+			}
+			p->gstate->clip_op = NO_CLIP_OP;
+		}
+
 		fz_drop_path(ctx, p->path);
 		p->path = NULL;
 		p->path = fz_new_path(ctx);
@@ -1586,6 +1579,9 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type)
 	sd.p = p;
 	sd.segment = NULL;
 	sd.type = type;
+	if (p->gstate->clip_op != NO_CLIP_OP)
+		sd.type += FZ_CULL_CLIP_PATH_FILL - FZ_CULL_PATH_FILL;
+	sd.flush = flush;
 	fz_try(ctx)
 	{
 		fz_walk_path(ctx, p->path, &segmenter, &sd);
@@ -1596,6 +1592,26 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type)
 	fz_catch(ctx)
 		fz_rethrow(ctx);
 
+	if (sd.any_sent != 0 && p->gstate->clip_op != NO_CLIP_OP)
+	{
+		fz_rect r;
+
+		/* ctm has always been flushed by now. */
+		r = fz_bound_path(ctx, p->path, NULL, p->gstate->sent.ctm);
+		p->gstate->clip_rect = fz_intersect_rect(p->gstate->clip_rect, r);
+		if (p->gstate->clip_op == CLIP_W)
+		{
+			if (p->chain->op_W)
+				p->chain->op_W(ctx, p->chain);
+		}
+		else
+		{
+			if (p->chain->op_Wstar)
+				p->chain->op_Wstar(ctx, p->chain);
+		}
+		p->gstate->clip_op = NO_CLIP_OP;
+	}
+
 	fz_drop_path(ctx, p->path);
 	p->path = NULL;
 	p->path = fz_new_path(ctx);
@@ -1612,10 +1628,9 @@ pdf_filter_S(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_STROKE, FLUSH_STROKE))
 		return;
 
-	filter_flush(ctx, p, FLUSH_STROKE);
 	if (p->chain->op_S)
 		p->chain->op_S(ctx, p->chain);
 }
@@ -1628,10 +1643,9 @@ pdf_filter_s(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_STROKE, FLUSH_STROKE))
 		return;
 
-	filter_flush(ctx, p, FLUSH_STROKE);
 	if (p->chain->op_s)
 		p->chain->op_s(ctx, p->chain);
 }
@@ -1644,10 +1658,9 @@ pdf_filter_F(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL, FLUSH_FILL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_FILL);
 	if (p->chain->op_F)
 		p->chain->op_F(ctx, p->chain);
 }
@@ -1660,10 +1673,9 @@ pdf_filter_f(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL, FLUSH_FILL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_FILL);
 	if (p->chain->op_f)
 		p->chain->op_f(ctx, p->chain);
 }
@@ -1676,10 +1688,9 @@ pdf_filter_fstar(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL, FLUSH_FILL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_FILL);
 	if (p->chain->op_fstar)
 		p->chain->op_fstar(ctx, p->chain);
 }
@@ -1692,10 +1703,9 @@ pdf_filter_B(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE, FLUSH_ALL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_ALL);
 	if (p->chain->op_B)
 		p->chain->op_B(ctx, p->chain);
 }
@@ -1708,10 +1718,9 @@ pdf_filter_Bstar(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE, FLUSH_ALL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_ALL);
 	if (p->chain->op_Bstar)
 		p->chain->op_Bstar(ctx, p->chain);
 }
@@ -1724,10 +1733,9 @@ pdf_filter_b(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE, FLUSH_ALL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_ALL);
 	if (p->chain->op_b)
 		p->chain->op_b(ctx, p->chain);
 }
@@ -1740,10 +1748,9 @@ pdf_filter_bstar(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE))
+	if (cull_path(ctx, p, FZ_CULL_PATH_FILL_STROKE, FLUSH_ALL))
 		return;
 
-	filter_flush(ctx, p, FLUSH_ALL);
 	if (p->chain->op_bstar)
 		p->chain->op_bstar(ctx, p->chain);
 }
@@ -1756,11 +1763,9 @@ pdf_filter_n(fz_context *ctx, pdf_processor *proc)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	fz_drop_path(ctx, p->path);
-	p->path = NULL;
-	p->path = fz_new_path(ctx);
+	if (cull_path(ctx, p, FZ_CULL_PATH_DROP, FLUSH_ALL))
+		return;
 
-	filter_flush(ctx, p, FLUSH_CTM);
 	if (p->chain->op_n)
 		p->chain->op_n(ctx, p->chain);
 }
@@ -1771,46 +1776,26 @@ static void
 pdf_filter_W(fz_context *ctx, pdf_processor *proc)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	fz_rect r;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	r = fz_bound_path(ctx, p->path, NULL, p->gstate->pending.ctm);
-	p->gstate->clip_rect = fz_intersect_rect(p->gstate->clip_rect, r);
-
-	if (fz_is_empty_rect(p->gstate->clip_rect))
-		return;
-
-	if (cull_path(ctx, p, FZ_CULL_CLIP_PATH))
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_W)
-		p->chain->op_W(ctx, p->chain);
+	/* This operator does nothing when it is seen. It just affects the
+	 * behaviour of the subsequent path drawing operation. */
+	p->gstate->clip_op = CLIP_W;
 }
 
 static void
 pdf_filter_Wstar(fz_context *ctx, pdf_processor *proc)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	fz_rect r;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	r = fz_bound_path(ctx, p->path, NULL, p->gstate->pending.ctm);
-	p->gstate->clip_rect = fz_intersect_rect(p->gstate->clip_rect, r);
-
-	if (fz_is_empty_rect(p->gstate->clip_rect))
-		return;
-
-	if (cull_path(ctx, p, FZ_CULL_CLIP_PATH))
-		return;
-
-	filter_flush(ctx, p, FLUSH_CTM);
-	if (p->chain->op_Wstar)
-		p->chain->op_Wstar(ctx, p->chain);
+	/* This operator does nothing when it is seen. It just affects the
+	 * behaviour of the subsequent path drawing operation. */
+	p->gstate->clip_op = CLIP_Wstar;
 }
 
 /* text objects */
@@ -2828,6 +2813,14 @@ pdf_sanitize_pop_resources(fz_context *ctx, pdf_processor *proc)
 	return pdf_processor_pop_resources(ctx, p->chain);
 }
 
+static void
+pdf_reset_sanitize_processor(fz_context *ctx, pdf_processor *proc)
+{
+	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
+
+	pdf_reset_processor(ctx, p->chain);
+}
+
 pdf_processor *
 pdf_new_sanitize_filter(
 	fz_context *ctx,
@@ -2843,6 +2836,7 @@ pdf_new_sanitize_filter(
 
 	proc->super.close_processor = pdf_close_sanitize_processor;
 	proc->super.drop_processor = pdf_drop_sanitize_processor;
+	proc->super.reset_processor = pdf_reset_sanitize_processor;
 
 	proc->super.push_resources = pdf_sanitize_push_resources;
 	proc->super.pop_resources = pdf_sanitize_pop_resources;
@@ -2990,6 +2984,7 @@ pdf_new_sanitize_filter(
 		proc->gstate->sent.stroke.linewidth = 1;
 		proc->gstate->sent.stroke.miterlimit = 10;
 		proc->gstate->clip_rect = fz_infinite_rect;
+		proc->gstate->clip_op = NO_CLIP_OP;
 	}
 	fz_catch(ctx)
 	{
